--- /dev/null	Mon Aug 27 16:56:41 2012
+++ new/usr/src/uts/common/os/seqlock.c	Mon Aug 27 16:56:41 2012
@@ -0,0 +1,66 @@
+/*
+ * This file and its contents are supplied under the terms of the
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may only use this file in accordance with the terms of version
+ * 1.0 of the CDDL.
+ *
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.illumos.org/license/CDDL.
+ */
+/*
+ * Copyright 2012 Nexenta Systems, Inc.  All rights reserved.
+ *
+ * seqlock.c - Implementation of sequence lock algorithm.
+ */
+#include <sys/types.h>
+#include <sys/ddi.h>
+#include <sys/mutex.h>
+#include <sys/cpu.h>		/* for SMT_PAUSE */
+#include <sys/cpuvar.h>		/* for kpreempt */
+#include <sys/seqlock.h>
+
+void
+seq_init(seqlock_t *seqlock)
+{
+	seqlock->sequence = 0;
+	mutex_init(&seqlock->lock, NULL, MUTEX_DEFAULT, NULL);
+}
+
+void
+seq_destroy(seqlock_t *seqlock)
+{
+	mutex_destroy(&seqlock->lock);
+}
+
+int
+seq_read_begin(volatile seqlock_t *seqlock)
+{
+	int seq;
+again:
+	seq = seqlock->sequence;
+	if (seq & 0x1) {
+		SMT_PAUSE();
+		goto again;
+	}
+	membar_consumer();
+	return (seq);
+}
+
+void
+seq_write_lock(seqlock_t *seqlock)
+{
+	mutex_enter(&seqlock->lock);
+	kpreempt_disable();
+	seqlock->sequence++;
+	membar_producer();
+}
+
+void
+seq_write_unlock(seqlock_t *seqlock)
+{
+	seqlock->sequence++;
+	membar_producer();
+	kpreempt_enable();
+	mutex_exit(&seqlock->lock);
+}
